#
# Autogenerated by Thrift Compiler (0.8.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
package EDAMNoteStore::Constants;
require 5.6.0;
use strict;
use warnings;
use Thrift;


1;
