package Common;
use base 'Exporter';

our @EXPORT_OK = qw($config);

our $config = {
    authentication_token      => 'S=s1:U=17f6f:E=14572b1de11:C=13e1b00b214:P=1cd:A=en-devtoken:V=2:H=7d28e083d0ac4c4597743241eb8e4292'
};
